@@ -28,7 +28,7 @@
  * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#import <objc/Object.h>
+#import "object.h"
 #include <stdarg.h>
 #include "errcode.h"
 #include "say.h"
@@ -38,16 +38,32 @@
  * Note: implements garbage collection (see +alloc
  * implementation).
  */
-@interface tnt_Exception: Object {
+@interface tnt_Exception: tnt_Object {
 	@public
 		const char *file;
 		unsigned line;
 }
-
 + (id) alloc;
 @end
 
 
+/** Internal error resulting from a failed system call.
+ */
+@interface SystemError: tnt_Exception {
+	@public
+		/* errno code */
+		int errnum;
+		/* error description */
+		char errmsg[TNT_ERRMSG_MAX];
+}
+
+- (id) init: (const char *)msg, ...;
+- (id) init: (int)errnum_arg: (const char *)format, ...;
+- (id) init: (int)errnum_arg: (const char *)format: (va_list)ap;
+- (void) log;
+@end
+
+
 /** Errors that should make it to the client.
  */
 @interface ClientError: tnt_Exception {
