@@ -2,17 +2,13 @@
 -- A run-time error will be raised on attempt to change
 -- table members.
 local function create_const_table(table)
-    return setmetatable ({}, {
-			     __index = table,
-			     __newindex = function(table_arg,
-						   name_arg,
-						   value_arg)
-				 error("attempting to change constant " ..
-				       tostring(name_arg) ..
-				       " to "
-				       .. tostring(value_arg), 2)
-			     end
-			     })
+    local function newindex(table, name, value)
+        error("Attempt to change constant "..tostring(name)..
+              " to "..tostring(value))
+    end
+    return setmetatable({}, { __index = table,
+                              __newindex = newindex,
+                              __metatable = false })
 end
 
 --- box flags
@@ -21,8 +17,6 @@ box.flags = create_const_table(
         BOX_RETURN_TUPLE = 0x01,
         BOX_ADD = 0x02,
         BOX_REPLACE = 0x04,
-        BOX_NOT_STORE = 0x10,
-        BOX_GC_TXN = 0x20
     })
 
 --
@@ -115,14 +109,19 @@ end
 --
 function box.update(space, key, format, ...)
     local op_count = select('#', ...)/2
-    return box.process(19,
-                       box.pack('iiipi'..format,
-                                  space,
-                                  1, -- flags, BOX_RETURN_TUPLE
-                                  1, -- primary key part count
-                                  key, -- primary key
-                                  op_count, -- op count
-                                  ...))
+    if type(key) == 'table' then
+        part_count = #key
+        return box.process(19,
+                    box.pack('iii'..string.rep('p', part_count),
+                        space, box.flags.BOX_RETURN_TUPLE, part_count,
+                        unpack(key))..
+                    box.pack('i'..format, op_count, ...))
+    else
+        return box.process(19,
+                    box.pack('iiipi'..format,
+                        space, box.flags.BOX_RETURN_TUPLE, 1,
+                        key, op_count, ...))
+    end
 end
 
 box.upd = {}
@@ -254,12 +253,25 @@ function box.on_reload_configuration()
     index_mt.max = function(index) return index.idx:max() end
     -- iteration
     index_mt.pairs = function(index)
-        return index.idx.next, index.idx, nil end
+        return index.idx.next, index.idx, nil
+    end
     --
     index_mt.next = function(index, ...)
-        return index.idx:next(...) end
+        return index.idx:next(...)
+    end
     index_mt.prev = function(index, ...)
-        return index.idx:prev(...) end
+        return index.idx:prev(...)
+    end
+    index_mt.next_equal = function(index, ...)
+        return index.idx:next_equal(...)
+    end
+    index_mt.prev_equal = function(index, ...)
+        return index.idx:prev_equal(...)
+    end
+    -- index subtree size
+    index_mt.count = function(index, ...)
+        return index.idx:count(...)
+    end
     --
     index_mt.select_range = function(index, limit, ...)
         local range = {}
@@ -325,10 +337,4 @@ function box.on_reload_configuration()
     end
 end
 
--- security: nullify some of the most serious os.* holes
-os.execute = nil
-os.exit = nil
-os.rename = nil
-os.tmpname = nil
-os.remove = nil
-require = nil
+-- vim: set et ts=4 sts
