@@ -39,6 +39,7 @@
 #include <stat.h>
 #include <tarantool.h>
 #include <tarantool_lua.h>
+#include <recovery.h>
 #include TARANTOOL_CONFIG
 #include <tbuf.h>
 #include <util.h>
@@ -68,7 +69,7 @@ static const char *help =
 static const char *unknown_command = "unknown command. try typing help." CRLF;
 
 
-#line 72 "src/admin.m"
+#line 73 "src/admin.m"
 static const int admin_start = 1;
 static const int admin_first_final = 135;
 static const int admin_error = 0;
@@ -76,7 +77,7 @@ static const int admin_error = 0;
 static const int admin_en_main = 1;
 
 
-#line 71 "src/admin.rl"
+#line 72 "src/admin.rl"
 
 
 
@@ -112,6 +113,18 @@ static void
 tarantool_info(struct tbuf *out)
 {
 	tbuf_printf(out, "info:" CRLF);
+	tbuf_printf(out, "  version: \"%s\"" CRLF, tarantool_version());
+	tbuf_printf(out, "  uptime: %i" CRLF, (int)tarantool_uptime());
+	tbuf_printf(out, "  pid: %i" CRLF, getpid());
+	tbuf_printf(out, "  logger_pid: %i" CRLF, logger_pid);
+	tbuf_printf(out, "  lsn: %" PRIi64 CRLF,
+		    recovery_state->confirmed_lsn);
+	tbuf_printf(out, "  recovery_lag: %.3f" CRLF,
+		    recovery_state->remote ? 
+		    recovery_state->remote->recovery_lag : 0);
+	tbuf_printf(out, "  recovery_last_update: %.3f" CRLF,
+		    recovery_state->remote ?
+		    recovery_state->remote->recovery_last_update_tstamp :0);
 	mod_info(out);
 	const char *path = cfg_filename_fullpath;
 	if (path == NULL)
@@ -138,12 +151,12 @@ admin_dispatch(lua_State *L)
 	p = fiber->rbuf->data;
 
 	
-#line 142 "src/admin.m"
+#line 155 "src/admin.m"
 	{
 	cs = admin_start;
 	}
 
-#line 147 "src/admin.m"
+#line 160 "src/admin.m"
 	{
 	if ( p == pe )
 		goto _test_eof;
@@ -206,15 +219,15 @@ case 6:
 	}
 	goto st0;
 tr13:
-#line 241 "src/admin.rl"
+#line 254 "src/admin.rl"
 	{slab_validate(); ok(out);}
 	goto st135;
 tr20:
-#line 229 "src/admin.rl"
+#line 242 "src/admin.rl"
 	{return 0;}
 	goto st135;
 tr25:
-#line 156 "src/admin.rl"
+#line 169 "src/admin.rl"
 	{
 			start(out);
 			tbuf_append(out, help, strlen(help));
@@ -222,9 +235,9 @@ tr25:
 		}
 	goto st135;
 tr36:
-#line 215 "src/admin.rl"
+#line 228 "src/admin.rl"
 	{strend = p;}
-#line 162 "src/admin.rl"
+#line 175 "src/admin.rl"
 	{
 			strstart[strend-strstart]='\0';
 			start(out);
@@ -233,7 +246,7 @@ tr36:
 		}
 	goto st135;
 tr43:
-#line 169 "src/admin.rl"
+#line 182 "src/admin.rl"
 	{
 			if (reload_cfg(err))
 				fail(out, err);
@@ -242,11 +255,11 @@ tr43:
 		}
 	goto st135;
 tr67:
-#line 239 "src/admin.rl"
+#line 252 "src/admin.rl"
 	{coredump(60); ok(out);}
 	goto st135;
 tr76:
-#line 176 "src/admin.rl"
+#line 189 "src/admin.rl"
 	{
 			int ret = snapshot(NULL, 0);
 
@@ -261,9 +274,9 @@ tr76:
 		}
 	goto st135;
 tr98:
-#line 225 "src/admin.rl"
+#line 238 "src/admin.rl"
 	{ state = false; }
-#line 189 "src/admin.rl"
+#line 202 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -275,9 +288,9 @@ tr98:
 		}
 	goto st135;
 tr101:
-#line 224 "src/admin.rl"
+#line 237 "src/admin.rl"
 	{ state = true; }
-#line 189 "src/admin.rl"
+#line 202 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -289,7 +302,7 @@ tr101:
 		}
 	goto st135;
 tr117:
-#line 132 "src/admin.rl"
+#line 145 "src/admin.rl"
 	{
 			tarantool_cfg_iterator_t *i;
 			char *key, *value;
@@ -309,15 +322,15 @@ tr117:
 		}
 	goto st135;
 tr131:
-#line 232 "src/admin.rl"
+#line 245 "src/admin.rl"
 	{start(out); fiber_info(out); end(out);}
 	goto st135;
 tr137:
-#line 231 "src/admin.rl"
+#line 244 "src/admin.rl"
 	{start(out); tarantool_info(out); end(out);}
 	goto st135;
 tr146:
-#line 150 "src/admin.rl"
+#line 163 "src/admin.rl"
 	{
 			start(out);
 			errinj_info(out);
@@ -325,33 +338,33 @@ tr146:
 		}
 	goto st135;
 tr152:
-#line 235 "src/admin.rl"
+#line 248 "src/admin.rl"
 	{start(out); palloc_stat(out); end(out);}
 	goto st135;
 tr160:
-#line 234 "src/admin.rl"
+#line 247 "src/admin.rl"
 	{start(out); slab_stat(out); end(out);}
 	goto st135;
 tr164:
-#line 236 "src/admin.rl"
+#line 249 "src/admin.rl"
 	{start(out); stat_print(out);end(out);}
 	goto st135;
 st135:
 	if ( ++p == pe )
 		goto _test_eof135;
 case 135:
-#line 344 "src/admin.m"
+#line 357 "src/admin.m"
 	goto st0;
 tr14:
-#line 241 "src/admin.rl"
+#line 254 "src/admin.rl"
 	{slab_validate(); ok(out);}
 	goto st7;
 tr21:
-#line 229 "src/admin.rl"
+#line 242 "src/admin.rl"
 	{return 0;}
 	goto st7;
 tr26:
-#line 156 "src/admin.rl"
+#line 169 "src/admin.rl"
 	{
 			start(out);
 			tbuf_append(out, help, strlen(help));
@@ -359,9 +372,9 @@ tr26:
 		}
 	goto st7;
 tr37:
-#line 215 "src/admin.rl"
+#line 228 "src/admin.rl"
 	{strend = p;}
-#line 162 "src/admin.rl"
+#line 175 "src/admin.rl"
 	{
 			strstart[strend-strstart]='\0';
 			start(out);
@@ -370,7 +383,7 @@ tr37:
 		}
 	goto st7;
 tr44:
-#line 169 "src/admin.rl"
+#line 182 "src/admin.rl"
 	{
 			if (reload_cfg(err))
 				fail(out, err);
@@ -379,11 +392,11 @@ tr44:
 		}
 	goto st7;
 tr68:
-#line 239 "src/admin.rl"
+#line 252 "src/admin.rl"
 	{coredump(60); ok(out);}
 	goto st7;
 tr77:
-#line 176 "src/admin.rl"
+#line 189 "src/admin.rl"
 	{
 			int ret = snapshot(NULL, 0);
 
@@ -398,9 +411,9 @@ tr77:
 		}
 	goto st7;
 tr99:
-#line 225 "src/admin.rl"
+#line 238 "src/admin.rl"
 	{ state = false; }
-#line 189 "src/admin.rl"
+#line 202 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -412,9 +425,9 @@ tr99:
 		}
 	goto st7;
 tr102:
-#line 224 "src/admin.rl"
+#line 237 "src/admin.rl"
 	{ state = true; }
-#line 189 "src/admin.rl"
+#line 202 "src/admin.rl"
 	{
 			strstart[strend-strstart] = '\0';
 			if (errinj_set_byname(strstart, state)) {
@@ -426,7 +439,7 @@ tr102:
 		}
 	goto st7;
 tr118:
-#line 132 "src/admin.rl"
+#line 145 "src/admin.rl"
 	{
 			tarantool_cfg_iterator_t *i;
 			char *key, *value;
@@ -446,15 +459,15 @@ tr118:
 		}
 	goto st7;
 tr132:
-#line 232 "src/admin.rl"
+#line 245 "src/admin.rl"
 	{start(out); fiber_info(out); end(out);}
 	goto st7;
 tr138:
-#line 231 "src/admin.rl"
+#line 244 "src/admin.rl"
 	{start(out); tarantool_info(out); end(out);}
 	goto st7;
 tr147:
-#line 150 "src/admin.rl"
+#line 163 "src/admin.rl"
 	{
 			start(out);
 			errinj_info(out);
@@ -462,22 +475,22 @@ tr147:
 		}
 	goto st7;
 tr153:
-#line 235 "src/admin.rl"
+#line 248 "src/admin.rl"
 	{start(out); palloc_stat(out); end(out);}
 	goto st7;
 tr161:
-#line 234 "src/admin.rl"
+#line 247 "src/admin.rl"
 	{start(out); slab_stat(out); end(out);}
 	goto st7;
 tr165:
-#line 236 "src/admin.rl"
+#line 249 "src/admin.rl"
 	{start(out); stat_print(out);end(out);}
 	goto st7;
 st7:
 	if ( ++p == pe )
 		goto _test_eof7;
 case 7:
-#line 481 "src/admin.m"
+#line 494 "src/admin.m"
 	if ( (*p) == 10 )
 		goto st135;
 	goto st0;
@@ -630,28 +643,28 @@ case 23:
 	}
 	goto tr33;
 tr33:
-#line 215 "src/admin.rl"
+#line 228 "src/admin.rl"
 	{strstart = p;}
 	goto st24;
 st24:
 	if ( ++p == pe )
 		goto _test_eof24;
 case 24:
-#line 641 "src/admin.m"
+#line 654 "src/admin.m"
 	switch( (*p) ) {
 		case 10: goto tr36;
 		case 13: goto tr37;
 	}
 	goto st24;
 tr34:
-#line 215 "src/admin.rl"
+#line 228 "src/admin.rl"
 	{strstart = p;}
 	goto st25;
 st25:
 	if ( ++p == pe )
 		goto _test_eof25;
 case 25:
-#line 655 "src/admin.m"
+#line 668 "src/admin.m"
 	switch( (*p) ) {
 		case 10: goto tr36;
 		case 13: goto tr37;
@@ -1101,28 +1114,28 @@ case 73:
 		goto tr91;
 	goto st0;
 tr91:
-#line 223 "src/admin.rl"
+#line 236 "src/admin.rl"
 	{ strstart = p; }
 	goto st74;
 st74:
 	if ( ++p == pe )
 		goto _test_eof74;
 case 74:
-#line 1112 "src/admin.m"
+#line 1125 "src/admin.m"
 	if ( (*p) == 32 )
 		goto tr92;
 	if ( 33 <= (*p) && (*p) <= 126 )
 		goto st74;
 	goto st0;
 tr92:
-#line 223 "src/admin.rl"
+#line 236 "src/admin.rl"
 	{ strend = p; }
 	goto st75;
 st75:
 	if ( ++p == pe )
 		goto _test_eof75;
 case 75:
-#line 1126 "src/admin.m"
+#line 1139 "src/admin.m"
 	switch( (*p) ) {
 		case 32: goto st75;
 		case 111: goto st76;
@@ -1814,7 +1827,7 @@ case 134:
 	_out: {}
 	}
 
-#line 247 "src/admin.rl"
+#line 260 "src/admin.rl"
 
 
 	tbuf_ltrim(fiber->rbuf, (void *)pe - (void *)fiber->rbuf->data);
