@@ -148,7 +148,7 @@ fiber_wakeup(struct fiber *f)
 void
 fiber_cancel(struct fiber *f)
 {
-	assert(fiber->fid != 0);
+	assert(f->fid != 0);
 	assert(!(f->flags & FIBER_CANCEL));
 
 	f->flags |= FIBER_CANCEL;
@@ -450,7 +450,7 @@ fiber_loop(void *data __attribute__((unused)))
 			say_info("fiber `%s': exiting", fiber->name);
 		}
 		@catch (id e) {
-			say_error("fiber `%s': exception `%s'", fiber->name, [e name]);
+			say_error("fiber `%s': exception `%s'", fiber->name, object_getClassName(e));
 			panic("fiber `%s': exiting", fiber->name);
 		}
 		fiber_close();
@@ -587,6 +587,7 @@ fiber_close(void)
 	fiber->fd = -1;
 	fiber->has_peer = false;
 	fiber->peer_name[0] = 0;
+	tbuf_reset(fiber->rbuf);
 
 	return r;
 }
