@@ -37,7 +37,12 @@
 #include <sys/uio.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <limits.h>
+#include <fcntl.h>
+
 #include "fiber.h"
+#include "recovery.h"
+#include "log_io.h"
 
 /** Replication topology
  * ----------------------
@@ -136,7 +141,7 @@ replication_relay_recv(struct ev_io *w, int revents);
 
 /** Send a single row to the client. */
 static int
-replication_relay_send_row(struct recovery_state *r __attribute__((unused)), struct tbuf *t);
+replication_relay_send_row(struct tbuf *t);
 
 
 /*
@@ -225,6 +230,16 @@ replication_init()
 	fiber_call(acceptor);
 }
 
+int sock_set_blocking(int sock)
+{
+	int flags = fcntl(sock, F_GETFL, 0);
+	if (flags >= 0 && flags & O_NONBLOCK)
+		flags = fcntl(sock, F_SETFL, flags & ~O_NONBLOCK);
+	if (flags < 0)
+		say_syserror("fcntl");
+	return flags;
+}
+
 
 /*-----------------------------------------------------------------------------*/
 /* replication accept/sender fibers                                            */
@@ -256,7 +271,12 @@ acceptor_handler(void *data __attribute__((unused)))
 			}
 			panic_syserror("accept");
 		}
-
+		/*
+		 * Drop the O_NONBLOCK flag, which was possible
+		 * inherited from the accept fd (happens on
+		 * Darwin).
+		 */
+		sock_set_blocking(client_sock);
 		/* up SO_KEEPALIVE flag */
 		int keepalive = 1;
 		if (setsockopt(client_sock, SOL_SOCKET, SO_KEEPALIVE,
@@ -601,7 +621,7 @@ replication_relay_loop(int client_sock)
 
 	ver = tbuf_alloc(fiber->gc_pool);
 	tbuf_append(ver, &default_version, sizeof(default_version));
-	replication_relay_send_row(NULL, ver);
+	replication_relay_send_row(ver);
 
 	/* init libev events handlers */
 	ev_default_loop(0);
@@ -614,7 +634,7 @@ replication_relay_loop(int client_sock)
 	ev_io_start(&sock_read_ev);
 
 	/* Initialize the recovery process */
-	recovery_init(NULL, cfg.wal_dir, replication_relay_send_row,
+	recovery_init(cfg.snap_dir, cfg.wal_dir, replication_relay_send_row,
 		      INT32_MAX, "fsync_delay", 0,
 		      RECOVER_READONLY);
 
@@ -648,7 +668,7 @@ replication_relay_recv(struct ev_io *w, int __attribute__((unused)) revents)
 
 /** Send to row to client. */
 static int
-replication_relay_send_row(struct recovery_state *r __attribute__((unused)), struct tbuf *t)
+replication_relay_send_row(struct tbuf *t)
 {
 	u8 *data = t->data;
 	ssize_t bytes, len = t->size;
