@@ -47,12 +47,13 @@
 #include <fiber.h>
 #include <iproto.h>
 #include <latch.h>
-#include <log_io.h>
+#include <recovery.h>
 #include <crc32.h>
 #include <palloc.h>
 #include <salloc.h>
 #include <say.h>
 #include <stat.h>
+#include <limits.h>
 #include TARANTOOL_CONFIG
 #include <util.h>
 #include <third_party/gopt/gopt.h>
@@ -369,7 +370,7 @@ signal_init(void)
 	ev_signal_start(&sigs[3]);
 
 	atexit(signal_free);
-	tt_pthread_atfork(NULL, NULL, signal_reset);
+	(void) tt_pthread_atfork(NULL, NULL, signal_reset);
 }
 
 static void
@@ -443,6 +444,13 @@ error:
 void
 tarantool_free(void)
 {
+	/*
+	 * Got to be done prior to anything else, since GC 
+	 * handlers can refer to other subsystems (e.g. fibers).
+	 */
+	if (tarantool_L)
+		tarantool_lua_close(tarantool_L);
+
 	recovery_free();
 	stat_free();
 
@@ -466,8 +474,6 @@ tarantool_free(void)
 #ifdef HAVE_BFD
 	symbols_free();
 #endif
-	if (tarantool_L)
-		tarantool_lua_close(tarantool_L);
 }
 
 static void
@@ -493,7 +499,7 @@ main(int argc, char **argv)
 #ifndef HAVE_LIBC_STACK_END
 /*
  * GNU libc provides a way to get at the top of the stack. This
- * is, of course, not standard and doesn't work on non-GNU
+ * is, of course, not-standard and doesn't work on non-GNU
  * systems, such as FreeBSD. But as far as we're concerned, argv
  * is at the top of the main thread's stack, so save the address
  * of it.
@@ -546,7 +552,8 @@ main(int argc, char **argv)
 		printf("Tarantool/%s %s\n", mod_name, tarantool_version());
 		printf("Target: %s\n", BUILD_INFO);
 		printf("Build options: %s\n", BUILD_OPTIONS);
-		printf("CFLAGS:%s\n", BUILD_CFLAGS);
+		printf("Compiler: %s\n", COMPILER_INFO);
+		printf("CFLAGS:%s\n", COMPILER_CFLAGS);
 		return 0;
 	}
 
