@@ -28,7 +28,8 @@
  * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
-#include "tarantool.h"
+#include <tarantool_lua.h>
+#include <tarantool.h>
 
 #include "lua.h"
 #include "lauxlib.h"
@@ -45,12 +46,15 @@
 #include <ctype.h>
 #include TARANTOOL_CONFIG
 
-/** tarantool start-up file */
+/**
+ * tarantool start-up file
+ */
 #define TARANTOOL_LUA_INIT_SCRIPT "init.lua"
 
 struct lua_State *tarantool_L;
 
-/* Remember the output of the administrative console in the
+/**
+ * Remember the output of the administrative console in the
  * registry, to use with 'print'.
  */
 static void
@@ -64,26 +68,31 @@ tarantool_lua_set_out(struct lua_State *L, const struct tbuf *out)
 	lua_settable(L, LUA_REGISTRYINDEX);
 }
 
-/* dup out from parent to child L. Used in fiber_create */
-
-void
+/**
+ * dup out from parent to child L. Used in fiber_create
+ */
+static void
 tarantool_lua_dup_out(struct lua_State *L, struct lua_State *child_L)
 {
 	lua_pushthread(L);
 	lua_gettable(L, LUA_REGISTRYINDEX);
 	struct tbuf *out = (struct tbuf *) lua_topointer(L, -1);
-	lua_pop(L, 1); /* pop 'out' */
+	/* pop 'out' */
+	lua_pop(L, 1);
 	if (out)
 		tarantool_lua_set_out(child_L, out);
 }
 
 
-/** {{{ box Lua library: common functions
+/*
+ * {{{ box Lua library: common functions
  */
 
 const char *boxlib_name = "box";
 
-/** Pack our BER integer into luaL_Buffer */
+/**
+ * Pack our BER integer into luaL_Buffer
+ */
 static void
 luaL_addvarint32(luaL_Buffer *b, u32 u32)
 {
@@ -98,11 +107,13 @@ uint64_t
 tarantool_lua_tointeger64(struct lua_State *L, int idx)
 {
 	uint64_t result = 0;
+
 	switch (lua_type(L, idx)) {
 	case LUA_TNUMBER:
 		result = lua_tointeger(L, idx);
 		break;
-	case LUA_TSTRING: {
+	case LUA_TSTRING:
+	{
 		const char *arg = luaL_checkstring(L, idx);
 		char *arge;
 		errno = 0;
@@ -111,22 +122,26 @@ tarantool_lua_tointeger64(struct lua_State *L, int idx)
 			luaL_error(L, "lua_tointeger64: bad argument");
 		break;
 	}
-	case LUA_TCDATA: {
+	case LUA_TCDATA:
+	{
 		if (lua_type(L, idx) != LUA_TCDATA)
 			luaL_error(L, "lua_tointeger64: cdata expected");
 		GCcdata *cd = cdataV(L->base + (idx - 1));
 		if (cd->typeid != CTID_INT64 && cd->typeid != CTID_UINT64)
-			luaL_error(L, "lua_tointeger64: unsupported cdata type");
+			luaL_error(L,
+				   "lua_tointeger64: unsupported cdata type");
 		result = *(uint64_t*)cdataptr(cd);
 		break;
 	}
 	default:
 		luaL_error(L, "lua_tointeger64: unsupported type");
 	}
+
 	return result;
 }
 
-/* Convert box.pack() format specifier to Tarantool
+/**
+ * Convert box.pack() format specifier to Tarantool
  * binary protocol UPDATE opcode
  */
 static char format_to_opcode(char format)
@@ -174,7 +189,8 @@ lbox_pack(struct lua_State *L)
 {
 	luaL_Buffer b;
 	const char *format = luaL_checkstring(L, 1);
-	int i = 2; /* first arg comes second */
+	/* first arg comes second */
+	int i = 2;
 	int nargs = lua_gettop(L);
 	u32 u32buf;
 	u64 u64buf;
@@ -185,38 +201,37 @@ lbox_pack(struct lua_State *L)
 
 	while (*format) {
 		if (i > nargs)
-			luaL_error(L, "box.pack: argument count does not match the format");
+			luaL_error(L, "box.pack: argument count does not match "
+				   "the format");
 		switch (*format) {
 		case 'B':
 		case 'b':
 			/* signed and unsigned 8-bit integers */
 			u32buf = lua_tointeger(L, i);
 			if (u32buf > 0xff)
-				luaL_error(L, "box.pack: argument too big for 8-bit integer");
+				luaL_error(L, "box.pack: argument too big for "
+					   "8-bit integer");
 			luaL_addchar(&b, (char) u32buf);
 			break;
-		/* signed and unsigned 32-bit integers */
 		case 'I':
 		case 'i':
-		{
+			/* signed and unsigned 32-bit integers */
 			u32buf = lua_tointeger(L, i);
 			luaL_addlstring(&b, (char *) &u32buf, sizeof(u32));
 			break;
-		}
 		case 'L':
 		case 'l':
-		{
+			/* signed and unsigned 64-bit integers */
 			u64buf = tarantool_lua_tointeger64(L, i);
 			luaL_addlstring(&b, (char *) &u64buf, sizeof(u64));
 			break;
-		}
-		/* Perl 'pack' BER-encoded integer */
 		case 'w':
+			/* Perl 'pack' BER-encoded integer */
 			luaL_addvarint32(&b, lua_tointeger(L, i));
 			break;
-		/* A sequence of bytes */
 		case 'A':
 		case 'a':
+			/* A sequence of bytes */
 			str = luaL_checklstring(L, i, &size);
 			luaL_addlstring(&b, str, size);
 			break;
@@ -226,8 +241,7 @@ lbox_pack(struct lua_State *L)
 				u32buf = (u32) lua_tointeger(L, i);
 				str = (char *) &u32buf;
 				size = sizeof(u32);
-			} else
-			if (lua_type(L, i) == LUA_TCDATA) {
+			} else if (lua_type(L, i) == LUA_TCDATA) {
 				u64buf = tarantool_lua_tointeger64(L, i);
 				str = (char *) &u64buf;
 				size = sizeof(u64);
@@ -237,22 +251,31 @@ lbox_pack(struct lua_State *L)
 			luaL_addvarint32(&b, size);
 			luaL_addlstring(&b, str, size);
 			break;
-		case '=': /* update tuple set foo=bar */
-		case '+': /* set field+=val */
-		case '&': /* set field&=val */
-		case '|': /* set field|=val */
-		case '^': /* set field^=val */
-		case ':': /* splice */
-		case '#': /* delete field */
-		case '!': /* insert field */
-			u32buf= (u32) lua_tointeger(L, i); /* field no */
+		case '=':
+			/* update tuple set foo = bar */
+		case '+':
+			/* set field += val */
+		case '&':
+			/* set field & =val */
+		case '|':
+			/* set field |= val */
+		case '^':
+			/* set field ^= val */
+		case ':':
+			/* splice */
+		case '#':
+			/* delete field */
+		case '!':
+			/* insert field */
+			/* field no */
+			u32buf = (u32) lua_tointeger(L, i);
 			luaL_addlstring(&b, (char *) &u32buf, sizeof(u32));
 			luaL_addchar(&b, format_to_opcode(*format));
 			break;
 		default:
 			luaL_error(L, "box.pack: unsupported pack "
 				   "format specifier '%c'", *format);
-		} /* end switch */
+		}
 		i++;
 		format++;
 	}
@@ -287,7 +310,8 @@ static int
 lbox_unpack(struct lua_State *L)
 {
 	const char *format = luaL_checkstring(L, 1);
-	int i = 2; /* first arg comes second */
+	/* first arg comes second */
+	int i = 2;
 	int nargs = lua_gettop(L);
 	size_t size;
 	const char *str;
@@ -295,12 +319,15 @@ lbox_unpack(struct lua_State *L)
 
 	while (*format) {
 		if (i > nargs)
-			luaL_error(L, "box.unpack: argument count does not match the format");
+			luaL_error(L, "box.unpack: argument count does not "
+				   "match the format");
 		switch (*format) {
 		case 'i':
 			str = lua_tolstring(L, i, &size);
 			if (str == NULL || size != sizeof(u32))
-				luaL_error(L, "box.unpack('%c'): got %d bytes (expected: 4)", *format, (int) size);
+				luaL_error(L, "box.unpack('%c'): got %d bytes "
+					   "(expected: 4)", *format,
+					   (int) size);
 			u32buf = * (u32 *) str;
 			lua_pushnumber(L, u32buf);
 			break;
@@ -308,7 +335,9 @@ lbox_unpack(struct lua_State *L)
 		{
 			str = lua_tolstring(L, i, &size);
 			if (str == NULL || size != sizeof(u64))
-				luaL_error(L, "box.unpack('%c'): got %d bytes (expected: 8)", *format, (int) size);
+				luaL_error(L, "box.unpack('%c'): got %d bytes "
+					   "(expected: 8)", *format,
+					   (int) size);
 			GCcdata *cd = luaL_pushcdata(L, CTID_UINT64, 8);
 			uint64_t *u64buf = (uint64_t*)cdataptr(cd);
 			*u64buf = *(u64*)str;
@@ -317,24 +346,27 @@ lbox_unpack(struct lua_State *L)
 		default:
 			luaL_error(L, "box.unpack: unsupported pack "
 				   "format specifier '%c'", *format);
-		} /* end switch */
+		}
 		i++;
 		format++;
 	}
 	return i-2;
 }
 
-/** A descriptor for box methods */
-
+/**
+ * descriptor for box methods
+ */
 static const struct luaL_reg boxlib[] = {
 	{"pack", lbox_pack},
 	{"unpack", lbox_unpack},
 	{NULL, NULL}
 };
 
-/* }}} */
+/*
+ * }}}
+ */
 
-/** {{{ box.fiber Lua library: access to Tarantool/Box fibers
+/* {{{ box.fiber Lua library: access to Tarantool/Box fibers
  *
  * Each fiber can be running, suspended or dead.
  * A fiber is created (box.fiber.create()) suspended.
@@ -385,7 +417,9 @@ static const struct luaL_reg boxlib[] = {
 
 static const char *fiberlib_name = "box.fiber";
 
-/** Push a userdata for the given fiber onto Lua stack. */
+/**
+ * Push a userdata for the given fiber onto Lua stack.
+ */
 static void
 lbox_pushfiber(struct lua_State *L, struct fiber *f)
 {
@@ -396,39 +430,55 @@ lbox_pushfiber(struct lua_State *L, struct fiber *f)
 	luaL_getmetatable(L, fiberlib_name);
 	int top = lua_gettop(L);
 	lua_getfield(L, -1, "memoize");
-	if (lua_isnil(L, -1)) { /* first access - instantiate memoize */
-		lua_pop(L, 1);                  /* pop the nil */
-		lua_newtable(L);                /* create memoize table */
-		lua_newtable(L);                /* and a metatable */
-		lua_pushstring(L, "kv"); /* weak keys and values */
-		lua_setfield(L, -2, "__mode"); /* pops 'kv' */
-		lua_setmetatable(L, -2); /* pops the metatable */
-		lua_setfield(L, -2, "memoize"); /* assigns and pops memoize */
-		lua_getfield(L, -1, "memoize"); /* gets memoize back. */
+	if (lua_isnil(L, -1)) {
+		/* first access - instantiate memoize */
+		/* pop the nil */
+		lua_pop(L, 1);
+		/* create memoize table */
+		lua_newtable(L);
+		/* and a metatable */
+		lua_newtable(L);
+		/* weak keys and values */
+		lua_pushstring(L, "kv");
+		/* pops 'kv' */
+		lua_setfield(L, -2, "__mode");
+		/* pops the metatable */
+		lua_setmetatable(L, -2);
+		/* assigns and pops memoize */
+		lua_setfield(L, -2, "memoize");
+		/* gets memoize back. */
+		lua_getfield(L, -1, "memoize");
 		assert(! lua_isnil(L, -1));
 	}
 	/* Find out whether the fiber is  already in the memoize table. */
 	lua_pushlightuserdata(L, f);
 	lua_gettable(L, -2);
-	if (lua_isnil(L, -1)) { /* no userdata for fiber created so far */
-		lua_pop(L, 1);                  /* pop the nil */
-		lua_pushlightuserdata(L, f);    /* push the key back */
+	if (lua_isnil(L, -1)) {
+		/* no userdata for fiber created so far */
+		/* pop the nil */
+		lua_pop(L, 1);
+		/* push the key back */
+		lua_pushlightuserdata(L, f);
 		/* create a new userdata */
 		void **ptr = lua_newuserdata(L, sizeof(void *));
 		*ptr = f;
 		luaL_getmetatable(L, fiberlib_name);
 		lua_setmetatable(L, -2);
-		lua_settable(L, -3);            /* memoize it */
+		/* memoize it */
+		lua_settable(L, -3);
 		lua_pushlightuserdata(L, f);
-		lua_gettable(L, -2);            /* get it back */
+		/* get it back */
+		lua_gettable(L, -2);
 	}
 	/*
 	 * Here we have a userdata on top of the stack and
 	 * possibly some garbage just under the top. Move the
 	 * result to the beginning of the stack and clear the rest.
 	 */
-	lua_replace(L, top); /* moves the current top to the old top */
-	lua_settop(L, top); /* clears everything after the old top */
+	/* moves the current top to the old top */
+	lua_replace(L, top);
+	/* clears everything after the old top */
+	lua_settop(L, top);
 }
 
 static struct fiber *
@@ -503,7 +553,6 @@ lbox_fiber_gc(struct lua_State *L)
 	return 0;
 }
 
-
 static void
 box_lua_fiber_run(void *arg __attribute__((unused)))
 {
@@ -523,8 +572,10 @@ box_lua_fiber_run(void *arg __attribute__((unused)))
 	 */
 	@try {
 		lua_call(L, lua_gettop(L) - 1, LUA_MULTRET);
-		lua_pushboolean(L, true); /* push completion status */
-		lua_insert(L, 1); /* move 'true' to stack start */
+		/* push completion status */
+		lua_pushboolean(L, true);
+		/* move 'true' to stack start */
+		lua_insert(L, 1);
 	} @catch (ClientError *e) {
 		/*
 		 * Note: FiberCancelException passes through this
@@ -533,9 +584,12 @@ box_lua_fiber_run(void *arg __attribute__((unused)))
 		 * cancel a fiber which is not scheduled, and
 		 * cancel() is synchronous.
 		 */
-		lua_settop(L, 0); /* pop any possible garbage */
-		lua_pushboolean(L, false); /* completion status */
-		lua_pushstring(L, e->errmsg); /* error message */
+		/* pop any possible garbage */
+		lua_settop(L, 0);
+		/* completion status */
+		lua_pushboolean(L, false);
+		/* error message */
+		lua_pushstring(L, e->errmsg);
 	} @finally {
 		/*
 		 * If the coroutine has detached itself, collect
@@ -549,18 +603,20 @@ box_lua_fiber_run(void *arg __attribute__((unused)))
 static int
 lbox_fiber_create(struct lua_State *L)
 {
-        if (lua_gettop(L) != 1 || !lua_isfunction(L, 1))
-                luaL_error(L, "fiber.create(function): bad arguments");
-	if (fiber_checkstack()) {
+	if (lua_gettop(L) != 1 || !lua_isfunction(L, 1))
+		luaL_error(L, "fiber.create(function): bad arguments");
+	if (fiber_checkstack())
 		luaL_error(L, "fiber.create(function): recursion limit"
 			   " reached");
-	}
-	struct fiber *f= fiber_create("lua", -1, box_lua_fiber_run, NULL);
 
-	lua_pushlightuserdata(L, f); /* associate coro with fiber */
+	struct fiber *f = fiber_create("lua", -1, box_lua_fiber_run, NULL);
+
+	/* associate coro with fiber */
+	lua_pushlightuserdata(L, f);
 	struct lua_State *child_L = lua_newthread(L);
 	lua_settable(L, LUA_REGISTRYINDEX);
-	lua_xmove(L, child_L, 1); /* move the argument to the new coro */
+	/* move the argument to the new coro */
+	lua_xmove(L, child_L, 1);
 	lbox_pushfiber(L, f);
 	return 1;
 }
@@ -583,14 +639,15 @@ lbox_fiber_resume(struct lua_State *L)
 	/* Get the results */
 	nargs = lua_gettop(child_L);
 	lua_xmove(child_L, L, nargs);
-	if (f->f == 0) { /* The fiber is dead. */
-		/* Garbage collect the associated coro. */
+	if (f->f == 0) {
+		/* The fiber is dead. Garbage collect the associated coro. */
 		box_lua_fiber_clear_coro(L, f);
 	}
 	return nargs;
 }
 
-/** Yield the current fiber.
+/**
+ * Yield the current fiber.
  *
  * Yield control to the calling fiber -- if the fiber
  * is attached, or to sched otherwise.
@@ -606,7 +663,8 @@ lbox_fiber_yield(struct lua_State *L)
 	 * whatever arguments are taken.
 	 */
 	fiber_yield();
-	fiber_testcancel(); /* throws an error if we were cancelled. */
+	/* throws an error if we were cancelled. */
+	fiber_testcancel();
 	/*
 	 * Got resumed. Return whatever the caller has passed
 	 * to us with box.fiber.resume().
@@ -667,11 +725,13 @@ lbox_fiber_detach(struct lua_State *L)
 	fiber_wakeup(fiber);
 	/* Yield to the parent. */
 	fiber_yield();
-	fiber_testcancel(); /* check if we were cancelled. */
+	/* check if we were cancelled. */
+	fiber_testcancel();
 	return 0;
 }
 
-/** Yield to the sched fiber and sleep.
+/**
+ * Yield to the sched fiber and sleep.
  * @param[in]  amount of time to sleep (double)
  *
  * Only the current fiber can be made to sleep.
@@ -705,7 +765,8 @@ lbox_fiber_find(struct lua_State *L)
 	return 1;
 }
 
-/** Running and suspended fibers can be cancelled.
+/**
+ * Running and suspended fibers can be cancelled.
  * Zombie fibers can't.
  */
 static int
@@ -753,28 +814,26 @@ static const struct luaL_reg fiberlib[] = {
 	{NULL, NULL}
 };
 
-/* }}} */
-
 /*
- * This function exists because lua_tostring does not use
- * __tostring metamethod, and this metamethod has to be used
- * if we want to print Lua userdata correctly.
+ * }}}
  */
+
 const char *
 tarantool_lua_tostring(struct lua_State *L, int index)
 {
-	if (index < 0) /* we need an absolute index */
+	/* we need an absolute index */
+	if (index < 0)
 		index = lua_gettop(L) + index + 1;
 	lua_getglobal(L, "tostring");
 	lua_pushvalue(L, index);
-	lua_call(L, 1, 1); /* pops both "tostring" and its argument */
+	/* pops both "tostring" and its argument */
+	lua_call(L, 1, 1);
 	lua_replace(L, index);
 	return lua_tostring(L, index);
 }
 
-/*
- * Convert Lua stack to YAML and append to
- * the given tbuf.
+/**
+ * Convert Lua stack to YAML and append to the given tbuf.
  */
 static void
 tarantool_lua_printstack_yaml(struct lua_State *L, struct tbuf *out)
@@ -786,11 +845,12 @@ tarantool_lua_printstack_yaml(struct lua_State *L, struct tbuf *out)
 			int len = strlen(sz);
 			tbuf_printf(out, " - %-.*s\r\n", len - 3, sz);
 		} else
-			tbuf_printf(out, " - %s\r\n", tarantool_lua_tostring(L, i));
+			tbuf_printf(out, " - %s\r\n",
+				    tarantool_lua_tostring(L, i));
 	}
 }
 
-/*
+/**
  * A helper to serialize arguments of 'print' Lua built-in
  * to tbuf.
  */
@@ -830,14 +890,17 @@ lbox_print(struct lua_State *L)
 	lua_pushthread(L);
 	lua_gettable(L, LUA_REGISTRYINDEX);
 	struct tbuf *out = (struct tbuf *) lua_topointer(L, -1);
-	lua_pop(L, 1); /* pop 'out' */
+	/* pop 'out' */
+	lua_pop(L, 1);
 
-	if (out) { /* Administrative console */
+	if (out) {
+		/* Administrative console */
 		tarantool_lua_printstack(L, out);
 		/* Courtesy: append YAML's \r\n if it's not already there */
 		if (out->size < 2 || tbuf_str(out)[out->size-1] != '\n')
 			tbuf_printf(out, "\r\n");
-	} else { /* Add a message to the server log */
+	} else {
+		/* Add a message to the server log */
 		out = tbuf_alloc(fiber->gc_pool);
 		tarantool_lua_printstack(L, out);
 		say_info("%s", tbuf_str(out));
@@ -851,7 +914,6 @@ lbox_print(struct lua_State *L)
  *
  * See Lua documentation on 'pcall' for additional information.
  */
-
 static int
 lbox_pcall(struct lua_State *L)
 {
@@ -861,22 +923,27 @@ lbox_pcall(struct lua_State *L)
 	 */
 	@try {
 		lua_call(L, lua_gettop(L) - 1, LUA_MULTRET);
-		lua_pushboolean(L, true); /* push completion status */
-		lua_insert(L, 1); /* move 'true' to stack start */
+		/* push completion status */
+		lua_pushboolean(L, true);
+		/* move 'true' to stack start */
+		lua_insert(L, 1);
 	} @catch (ClientError *e) {
 		/*
 		 * Note: FiberCancelException passes through this
 		 * catch and thus leaves garbage on coroutine
 		 * stack.
 		 */
-		lua_settop(L, 0); /* pop any possible garbage */
-		lua_pushboolean(L, false); /* completion status */
-		lua_pushstring(L, e->errmsg); /* error message */
+		/* pop any possible garbage */
+		lua_settop(L, 0);
+		/* completion status */
+		lua_pushboolean(L, false);
+		/* error message */
+		lua_pushstring(L, e->errmsg);
 	}
 	return lua_gettop(L);
 }
 
-/*
+/**
  * Convert lua number or string to lua cdata 64bit number.
  */
 static int
@@ -886,7 +953,9 @@ lbox_tonumber64(struct lua_State *L)
 	return luaL_pushnumber64(L, result);
 }
 
-/** A helper to register a single type metatable. */
+/**
+ * A helper to register a single type metatable.
+ */
 void
 tarantool_lua_register_type(struct lua_State *L, const char *type_name,
 			    const struct luaL_Reg *methods)
@@ -936,7 +1005,8 @@ tarantool_lua_init()
 	lua_register(L, "pcall", lbox_pcall);
 	lua_register(L, "tonumber64", lbox_tonumber64);
 	L = mod_lua_init(L);
-	lua_settop(L, 0); /* clear possible left-overs of init */
+	/* clear possible left-overs of init */
+	lua_settop(L, 0);
 	return L;
 }
 
@@ -944,7 +1014,8 @@ void
 tarantool_lua_close(struct lua_State *L)
 {
 	luaL_unref(L, LUA_REGISTRYINDEX, ffi_ref);
-	lua_close(L); /* collects garbage, invoking userdata gc */
+	/* collects garbage, invoking userdata gc */
+	lua_close(L);
 }
 
 /**
@@ -960,7 +1031,8 @@ tarantool_lua_dostring(struct lua_State *L, const char *str)
 	tbuf_printf(buf, "%s%s", "return ", str);
 	int r = luaL_loadstring(L, tbuf_str(buf));
 	if (r) {
-		lua_pop(L, 1); /* pop the error message */
+		/* pop the error message */
+		lua_pop(L, 1);
 		r = luaL_loadstring(L, str);
 		if (r)
 			return r;
@@ -994,11 +1066,11 @@ tarantool_lua(struct lua_State *L,
 		tbuf_printf(out, "error: '%s'\r\n",
 			    luaL_gsub(L, lua_tostring(L, -1),
 				      "'", "''"));
-	}
-	else {
+	} else {
 		tarantool_lua_printstack_yaml(L, out);
 	}
-	lua_settop(L, 0); /* clear the stack from return values. */
+	/* clear the stack from return values. */
+	lua_settop(L, 0);
 }
 
 /**
@@ -1014,11 +1086,12 @@ is_string(const char *str)
 	    return true;
 	char *endptr;
 	double r = strtod(str, &endptr);
-	(void) r;  /* -Wunused-result warning suppression */
+	/* -Wunused-result warning suppression */
+	(void) r;
 	return *endptr != '\0';
 }
 
-/*
+/**
  * Make a new configuration available in Lua.
  * We could perhaps make Lua bindings to access the C
  * structure in question, but for now it's easier and just
@@ -1034,15 +1107,16 @@ tarantool_lua_load_cfg(struct lua_State *L, struct tarantool_cfg *cfg)
 	luaL_buffinit(L, &b);
 	tarantool_cfg_iterator_t *i = tarantool_cfg_iterator_init();
 	luaL_addstring(&b,
-"box.cfg = {}\n"
-"setmetatable(box.cfg, {})\n"
-"box.space = {}\n"
-"setmetatable(box.space, getmetatable(box.cfg))\n"
-"getmetatable(box.space).__index = function(table, index)\n"
-"  table[index] = {}\n"
-"  setmetatable(table[index], getmetatable(table))\n"
-"  return rawget(table, index)\n"
-"end\n");
+		       "box.cfg = {}\n"
+		       "setmetatable(box.cfg, {})\n"
+		       "box.space = {}\n"
+		       "setmetatable(box.space, getmetatable(box.cfg))\n"
+		       "getmetatable(box.space).__index = "
+		       "function(table, index)\n"
+		       "  table[index] = {}\n"
+		       "  setmetatable(table[index], getmetatable(table))\n"
+		       "  return rawget(table, index)\n"
+		       "end\n");
 	while ((key = tarantool_cfg_iterator_next(i, cfg, &value)) != NULL) {
 		if (value == NULL)
 			continue;
@@ -1059,13 +1133,15 @@ tarantool_lua_load_cfg(struct lua_State *L, struct tarantool_cfg *cfg)
 		free(value);
 	}
 	luaL_addstring(&b,
-"getmetatable(box.cfg).__newindex = function(table, index)\n"
-"  error('Attempt to modify a read-only table')\n"
-"end\n"
-"getmetatable(box.cfg).__index = nil\n"
-"if type(box.on_reload_configuration) == 'function' then\n"
-"  box.on_reload_configuration()\n"
-"end\n");
+		       "getmetatable(box.cfg).__newindex = "
+		       "function(table, index)\n"
+		       "  error('Attempt to modify a read-only table')\n"
+		       "end\n"
+		       "getmetatable(box.cfg).__index = nil\n"
+		       "if type(box.on_reload_configuration) == 'function' "
+		       "then\n"
+		       "  box.on_reload_configuration()\n"
+		       "end\n");
 	luaL_pushresult(&b);
 	if (luaL_loadstring(L, lua_tostring(L, -1)) != 0 ||
 	    lua_pcall(L, 0, 0, 0) != 0) {
@@ -1075,38 +1151,75 @@ tarantool_lua_load_cfg(struct lua_State *L, struct tarantool_cfg *cfg)
 }
 
 /**
- * Load start-up file routine
+ * Load start-up file routine.
  */
 static void
 load_init_script(void *L_ptr)
 {
 	struct lua_State *L = (struct lua_State *) L_ptr;
-	struct stat st;
-	/* checking that Lua start-up file exist. */
-	if (stat(TARANTOOL_LUA_INIT_SCRIPT, &st)) {
-		/*
-		 * File doesn't exist. It's OK, tarantool may not have
-		 * start-up file.
-		 */
-		return;
-	}
 
-	/* execute start-up file */
-	if (tarantool_lua_dofile(L, TARANTOOL_LUA_INIT_SCRIPT))
+	char path[PATH_MAX + 1];
+	snprintf(path, PATH_MAX, "%s/%s",
+		 cfg.script_dir, TARANTOOL_LUA_INIT_SCRIPT);
+
+
+	if (access(path, F_OK) == 0) {
+		say_info("loading %s", path);
+		/* Execute the init file. */
+		if (tarantool_lua_dofile(L, path))
+			panic("%s", lua_tostring(L, -1));
+	}
+	/*
+	 * The file doesn't exist. It's OK, tarantool may
+	 * have no init file.
+	 */
+}
+
+/**
+ * Unset functions in the Lua state which can be used to
+ * execute external programs or otherwise introduce a breach
+ * in security.
+ *
+ * @param L is a Lua State.
+ */
+static void
+tarantool_lua_sandbox(struct lua_State *L)
+{
+	/*
+	 * Unset some functions for security reasons:
+	 * 1. Some os.* functions (like os.execute, os.exit, etc..)
+	 * 2. require(), since it can be used to provide access to ffi
+	 * or anything else we unset in 1.
+	 */
+	int result = tarantool_lua_dostring(L,
+					    "os.execute = nil\n"
+					    "os.exit = nil\n"
+					    "os.rename = nil\n"
+					    "os.tmpname = nil\n"
+					    "os.remove = nil\n"
+					    "require = nil\n");
+	if (result)
 		panic("%s", lua_tostring(L, -1));
 }
 
-void tarantool_lua_load_init_script(struct lua_State *L)
+void
+tarantool_lua_load_init_script(struct lua_State *L)
 {
 	/*
 	 * init script can call box.fiber.yield (including implicitly via
-	 * box.insert, box.update, etc...) but yield which called in sched
-	 * fiber will crash the server. That why, to avoid the problem, we must
-	 * run init script in to separate fiber.
+	 * box.insert, box.update, etc...) but box.fiber.yield() today,
+	 * when called from 'sched' fiber crashes the server.
+	 * To work this problem around we must run init script in
+	 * a separate fiber.
 	 */
 	struct fiber *loader = fiber_create(TARANTOOL_LUA_INIT_SCRIPT, -1,
 					    load_init_script, L);
 	fiber_call(loader);
+	/* Outside the startup file require() or ffi are not
+	 * allowed.
+	*/
+	tarantool_lua_sandbox(tarantool_L);
+
 }
 
 /*
