@@ -188,6 +188,25 @@ static void tt_tnt_sbuf(struct tt_test *test) {
 	tnt_stream_free(&s);
 }
 
+/* tuple set */
+static void tt_tnt_tuple_set(struct tt_test *test) {
+	char buf[75];
+	*((uint32_t*)buf) = 2; /* cardinality */
+	/* 4 + 1 + 5 + 1 + 64 = 75 */
+	uint32_t off = sizeof(uint32_t);
+	int esize = tnt_enc_size(5);
+	tnt_enc_write(buf + off, 5);
+	off += esize + 5;
+	esize = tnt_enc_size(64);
+	tnt_enc_write(buf + off, 64);
+	off += esize + 64;
+	struct tnt_tuple t;
+	tnt_tuple_init(&t);
+	TT_ASSERT(tnt_tuple_set(&t, buf, 70) == NULL);
+	TT_ASSERT(tnt_tuple_set(&t, buf, sizeof(buf)) != NULL);
+	tnt_tuple_free(&t);
+}
+
 /* iterator tuple */
 static void tt_tnt_iter1(struct tt_test *test) {
 	struct tnt_tuple *t = tnt_tuple(NULL, "%s%d%s", "foo", 123, "bar");
@@ -276,6 +295,236 @@ static void tt_tnt_iter3(struct tt_test *test) {
 	tnt_list_free(l);
 }
 
+/* marshal ping */
+static void tt_tnt_marshal_ping(struct tt_test *test) {
+	struct tnt_stream s;
+	tnt_buf(&s);
+	tnt_ping(&s);
+	tnt_ping(&s);
+	struct tnt_iter i;
+	tnt_iter_request(&i, &s);
+	TT_ASSERT(tnt_next(&i) == 1);
+	struct tnt_request *r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_PING);
+	TT_ASSERT(tnt_next(&i) == 1);
+	TT_ASSERT(r->h.type == TNT_OP_PING);
+	TT_ASSERT(tnt_next(&i) == 0);
+	tnt_iter_free(&i);
+	tnt_stream_free(&s);
+}
+
+/* marshal insert */
+static void tt_tnt_marshal_insert(struct tt_test *test) {
+	struct tnt_stream s;
+	tnt_buf(&s);
+	struct tnt_tuple t;
+	tnt_tuple_init(&t);
+	tnt_tuple(&t, "%s%d", "foo", 123);
+	tnt_insert(&s, 0, 0, &t);
+	tnt_insert(&s, 0, 0, &t);
+	struct tnt_iter i;
+	tnt_iter_request(&i, &s);
+	TT_ASSERT(tnt_next(&i) == 1);
+	struct tnt_request *r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_INSERT);
+	struct tnt_iter *f = tnt_field(NULL, &r->r.insert.t, 0);
+	TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+	TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+	TT_ASSERT(tnt_field(f, NULL, 1) != NULL);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 4);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 1);
+	TT_ASSERT(*(uint32_t*)TNT_IFIELD_DATA(f) == 123);
+	TT_ASSERT(tnt_next(&i) == 1);
+	r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_INSERT);
+	TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+	TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+	TT_ASSERT(tnt_field(f, NULL, 1) != NULL);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 4);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 1);
+	TT_ASSERT(*(uint32_t*)TNT_IFIELD_DATA(f) == 123);
+	TT_ASSERT(tnt_next(&i) == 0);
+	tnt_tuple_free(&t);
+	tnt_iter_free(&i);
+	tnt_stream_free(&s);
+}
+
+/* marshal delete */
+static void tt_tnt_marshal_delete(struct tt_test *test) {
+	struct tnt_stream s;
+	tnt_buf(&s);
+	struct tnt_tuple t;
+	tnt_tuple_init(&t);
+	tnt_tuple(&t, "%s", "foo");
+	tnt_delete(&s, 0, 0, &t);
+	tnt_delete(&s, 0, 0, &t);
+	struct tnt_iter i;
+	tnt_iter_request(&i, &s);
+	TT_ASSERT(tnt_next(&i) == 1);
+	struct tnt_request *r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_DELETE);
+	struct tnt_iter *f = tnt_field(NULL, &r->r.del.t, 0);
+	TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+	TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+	TT_ASSERT(tnt_next(&i) == 1);
+	r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_DELETE);
+	TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+	TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+	TT_ASSERT(tnt_next(&i) == 0);
+	tnt_tuple_free(&t);
+	tnt_iter_free(&i);
+	tnt_stream_free(&s);
+}
+
+/* marshal call */
+static void tt_tnt_marshal_call(struct tt_test *test) {
+	struct tnt_stream s;
+	tnt_buf(&s);
+	struct tnt_tuple t;
+	tnt_tuple_init(&t);
+	tnt_tuple(&t, "%s%d", "foo", 123);
+	tnt_call(&s, 0, "box.select", &t);
+	tnt_call(&s, 0, "box.select", &t);
+	struct tnt_iter i;
+	tnt_iter_request(&i, &s);
+	TT_ASSERT(tnt_next(&i) == 1);
+	struct tnt_request *r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_CALL);
+	TT_ASSERT(strcmp(r->r.call.proc, "box.select") == 0);
+	struct tnt_iter *f = tnt_field(NULL, &r->r.call.t, 0);
+	TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+	TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+	TT_ASSERT(tnt_field(f, NULL, 1) != NULL);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 4);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 1);
+	TT_ASSERT(*(uint32_t*)TNT_IFIELD_DATA(f) == 123);
+	TT_ASSERT(tnt_next(&i) == 1);
+	r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_CALL);
+	TT_ASSERT(strcmp(r->r.call.proc, "box.select") == 0);
+	TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+	TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+	TT_ASSERT(tnt_field(f, NULL, 1) != NULL);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 4);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 1);
+	TT_ASSERT(*(uint32_t*)TNT_IFIELD_DATA(f) == 123);
+	TT_ASSERT(tnt_next(&i) == 0);
+	tnt_tuple_free(&t);
+	tnt_iter_free(&i);
+	tnt_stream_free(&s);
+}
+
+/* marshal select */
+static void tt_tnt_marshal_select(struct tt_test *test) {
+	struct tnt_stream s;
+	tnt_buf(&s);
+	struct tnt_list list;
+	tnt_list_init(&list);
+	tnt_list(&list, tnt_tuple(NULL, "%s", "foo"),
+			tnt_tuple(NULL, "%s%d", "bar", 444),
+			tnt_tuple(NULL, "%s%d%d", "baz", 1, 2),
+			NULL);
+	tnt_select(&s, 0, 0, 0, 1, &list);
+	struct tnt_iter i;
+	tnt_iter_request(&i, &s);
+	TT_ASSERT(tnt_next(&i) == 1);
+	struct tnt_request *r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_SELECT);
+	struct tnt_iter il;
+	tnt_iter_list(&il, &r->r.select.l);
+	TT_ASSERT(tnt_next(&il) == 1);
+		struct tnt_tuple *t = TNT_ILIST_TUPLE(&il);
+		struct tnt_iter *f = tnt_field(NULL, t, 0);
+		TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+		TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+		TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+		TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+		tnt_iter_free(f);
+	TT_ASSERT(tnt_next(&il) == 1);
+		t = TNT_ILIST_TUPLE(&il);
+		f = tnt_field(NULL, t, 0);
+		TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+		TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+		TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+		TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "bar", 3) == 0);
+		TT_ASSERT(tnt_field(f, NULL, 1) != NULL);
+		TT_ASSERT(TNT_IFIELD_SIZE(f) == 4);
+		TT_ASSERT(TNT_IFIELD_IDX(f) == 1);
+		TT_ASSERT(*(uint32_t*)TNT_IFIELD_DATA(f) == 444);
+		tnt_iter_free(f);
+	TT_ASSERT(tnt_next(&il) == 1);
+		t = TNT_ILIST_TUPLE(&il);
+		f = tnt_field(NULL, t, 0);
+		TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+		TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+		TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+		TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "baz", 3) == 0);
+		TT_ASSERT(tnt_field(f, NULL, 1) != NULL);
+		TT_ASSERT(TNT_IFIELD_SIZE(f) == 4);
+		TT_ASSERT(TNT_IFIELD_IDX(f) == 1);
+		TT_ASSERT(*(uint32_t*)TNT_IFIELD_DATA(f) == 1);
+		TT_ASSERT(tnt_field(f, NULL, 2) != NULL);
+		TT_ASSERT(TNT_IFIELD_SIZE(f) == 4);
+		TT_ASSERT(TNT_IFIELD_IDX(f) == 2);
+		TT_ASSERT(*(uint32_t*)TNT_IFIELD_DATA(f) == 2);
+		tnt_iter_free(f);
+	TT_ASSERT(tnt_next(&il) == 0);
+	tnt_iter_free(&i);
+	tnt_iter_free(&il);
+	tnt_list_free(&list);
+	tnt_stream_free(&s);
+}
+
+/* marshal update */
+static void tt_tnt_marshal_update(struct tt_test *test) {
+	struct tnt_stream s, ops;
+	tnt_buf(&s);
+	tnt_buf(&ops);
+	struct tnt_tuple t;
+	tnt_tuple_init(&t);
+	tnt_tuple(&t, "%s", "foo");
+	tnt_update_assign(&ops, 444, "FOO", 3);
+	tnt_update_arith(&ops, 2, TNT_UPDATE_ADD, 7);
+	TT_ASSERT(tnt_update(&s, 0, 0, &t, &ops) > 0);
+	struct tnt_iter i;
+	tnt_iter_request(&i, &s);
+	TT_ASSERT(tnt_next(&i) == 1);
+	struct tnt_request *r = TNT_IREQUEST_PTR(&i);
+	TT_ASSERT(r->h.type == TNT_OP_UPDATE);
+	TT_ASSERT(r->r.update.opc == 2);
+	struct tnt_iter *f = tnt_field(NULL, &r->r.update.t, 0);
+	TT_ASSERT(tnt_field(f, NULL, 0) != NULL);
+	TT_ASSERT(TNT_IFIELD_IDX(f) == 0);
+	TT_ASSERT(TNT_IFIELD_SIZE(f) == 3);
+	TT_ASSERT(memcmp(TNT_IFIELD_DATA(f), "foo", 3) == 0);
+	TT_ASSERT(r->r.update.opv[0].op == TNT_UPDATE_ASSIGN);
+	TT_ASSERT(r->r.update.opv[0].field == 444);
+	TT_ASSERT(r->r.update.opv[0].size == 3);
+	TT_ASSERT(memcmp(r->r.update.opv[0].data, "FOO", 3) == 0);
+	TT_ASSERT(r->r.update.opv[1].op == TNT_UPDATE_ADD);
+	TT_ASSERT(r->r.update.opv[1].field == 2);
+	TT_ASSERT(r->r.update.opv[1].size == 4);
+	TT_ASSERT(*(uint32_t*)r->r.update.opv[1].data == 7);
+	TT_ASSERT(tnt_next(&i) == 0);
+	tnt_tuple_free(&t);
+	tnt_stream_free(&s);
+	tnt_stream_free(&ops);
+	tnt_iter_free(&i);
+}
+
 static struct tnt_stream net;
 
 /* network connection */
@@ -292,9 +541,9 @@ static void tt_tnt_net_ping(struct tt_test *test) {
 	TT_ASSERT(tnt_ping(&net) > 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_PING);
 	}
@@ -316,9 +565,9 @@ static void tt_tnt_net_insert(struct tt_test *test) {
 	tnt_tuple_free(&kv1);
 	tnt_tuple_free(&kv2);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->reqid == 777);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_INSERT);
@@ -339,9 +588,9 @@ static void tt_tnt_net_update(struct tt_test *test) {
 	tnt_stream_free(&ops);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_UPDATE);
 		TT_ASSERT(r->count == 1);
@@ -356,9 +605,9 @@ static void tt_tnt_net_select(struct tt_test *test) {
 	TT_ASSERT(tnt_flush(&net) > 0);
 	tnt_list_free(search);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_SELECT);
 		TT_ASSERT(r->count == 1);
@@ -396,9 +645,9 @@ static void tt_tnt_net_delete(struct tt_test *test) {
 	TT_ASSERT(tnt_flush(&net) > 0);
 	tnt_tuple_free(&k);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_DELETE);
 		TT_ASSERT(r->count == 1);
@@ -415,9 +664,9 @@ static void tt_tnt_net_call(struct tt_test *test) {
 	TT_ASSERT(tnt_flush(&net) > 0);
 	tnt_tuple_free(&args);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_CALL);
 		TT_ASSERT(r->count == 1);
@@ -433,9 +682,9 @@ static void tt_tnt_net_call_na(struct tt_test *test) {
 	TT_ASSERT(tnt_flush(&net) > 0);
 	tnt_tuple_free(&args);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code != 0);
 		TT_ASSERT(strcmp(r->error, "Illegal parameters, tuple field count is 0") == 0);
 	}
@@ -707,9 +956,9 @@ static void tt_tnt_sql_ping(struct tt_test *test) {
 	TT_ASSERT(tnt_query(&net, q, sizeof(q) - 1, &e) == 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_PING);
 	}
@@ -723,9 +972,9 @@ static void tt_tnt_sql_insert(struct tt_test *test) {
 	TT_ASSERT(tnt_query(&net, q, sizeof(q) - 1, &e) == 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_INSERT);
 		TT_ASSERT(r->count == 1);
@@ -758,9 +1007,9 @@ static void tt_tnt_sql_update(struct tt_test *test) {
 	TT_ASSERT(tnt_query(&net, q7, sizeof(q7) - 1, &e) == 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_UPDATE);
 		TT_ASSERT(r->count == 1);
@@ -775,9 +1024,9 @@ static void tt_tnt_sql_select(struct tt_test *test) {
 	TT_ASSERT(tnt_query(&net, q, sizeof(q) - 1, &e) == 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_SELECT);
 		TT_ASSERT(r->count == 2);
@@ -814,9 +1063,9 @@ static void tt_tnt_sql_select_limit(struct tt_test *test) {
 	TT_ASSERT(tnt_query(&net, q, sizeof(q) - 1, &e) == 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_SELECT);
 		TT_ASSERT(r->count == 0);
@@ -831,9 +1080,9 @@ static void tt_tnt_sql_delete(struct tt_test *test) {
 	TT_ASSERT(tnt_query(&net, q, sizeof(q) - 1, &e) == 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_DELETE);
 		TT_ASSERT(r->count == 1);
@@ -848,9 +1097,9 @@ static void tt_tnt_sql_call(struct tt_test *test) {
 	TT_ASSERT(tnt_query(&net, q, sizeof(q) - 1, &e) == 0);
 	TT_ASSERT(tnt_flush(&net) > 0);
 	struct tnt_iter i;
-	tnt_iter_stream(&i, &net);
+	tnt_iter_reply(&i, &net);
 	while (tnt_next(&i)) {
-		struct tnt_reply *r = TNT_ISTREAM_REPLY(&i);
+		struct tnt_reply *r = TNT_IREPLY_PTR(&i);
 		TT_ASSERT(r->code == 0);
 		TT_ASSERT(r->op == TNT_OP_CALL);
 		TT_ASSERT(r->count == 1);
@@ -871,10 +1120,18 @@ main(int argc, char * argv[])
 	tt_test(&t, "tuple2", tt_tnt_tuple2);
 	tt_test(&t, "list", tt_tnt_list);
 	tt_test(&t, "stream buffer", tt_tnt_sbuf);
+	tt_test(&t, "tuple set", tt_tnt_tuple_set);
 	tt_test(&t, "iterator tuple", tt_tnt_iter1);
 	tt_test(&t, "iterator tuple (single field)", tt_tnt_iter11);
 	tt_test(&t, "iterator tuple (tnt_field)", tt_tnt_iter2);
 	tt_test(&t, "iterator list", tt_tnt_iter3);
+	/* marshaling */
+	tt_test(&t, "marshaling ping", tt_tnt_marshal_ping);
+	tt_test(&t, "marshaling insert", tt_tnt_marshal_insert);
+	tt_test(&t, "marshaling delete", tt_tnt_marshal_delete);
+	tt_test(&t, "marshaling call", tt_tnt_marshal_call);
+	tt_test(&t, "marshaling select", tt_tnt_marshal_select);
+	tt_test(&t, "marshaling update", tt_tnt_marshal_update);
 	/* common operations */
 	tt_test(&t, "connect", tt_tnt_net_connect);
 	tt_test(&t, "ping", tt_tnt_net_ping);
